SET STATISTICS IO ON
SET STATISTICS TIME ON

USE AdventureWorks2012
GO

-- Demonstrates a simple Hash Join
SELECT
	p1.FirstName,
	p1.LastName,
	p2.PhoneNumber
FROM Person.Person p1
INNER JOIN Person.PersonPhone p2
ON p1.BusinessEntityID = p2.BusinessEntityID
GO

-- Create a new database
CREATE DATABASE HashSpills
GO

-- Use it
USE HashSpills
GO

-- Create a test table
CREATE TABLE Table1
(
	Column1 INT IDENTITY PRIMARY KEY,
	Column2 INT,
	Column3 CHAR(2000)
)
GO

-- Create a Non-Clustered Index on column Column2
CREATE NONCLUSTERED INDEX idxTable1_Column2 ON Table1(Column2)
GO

-- Insert 1500 records into Table1
SELECT TOP 1500 IDENTITY(INT, 1, 1) AS n INTO #Nums
FROM
master.dbo.syscolumns sc1

INSERT INTO Table1 (Column2, Column3)
SELECT n, REPlICATE('x', 2000) FROM #nums
DROP TABLE #nums
GO

-- Retrieve the inserted records
SELECT * FROM Table1
GO

-- Create a test table
CREATE TABLE Table2
(
	Column1 INT IDENTITY PRIMARY KEY,
	Column2 INT,
	Column3 CHAR(2000)
)
GO

-- Create a Non-Clustered Index on column Column2
CREATE NONCLUSTERED INDEX idxTable2_Column2 ON Table2(Column2)
GO

-- Insert 1500 records into Table1
SELECT TOP 1500 IDENTITY(INT, 1, 1) AS n INTO #Nums
FROM
master.dbo.syscolumns sc1

INSERT INTO Table2 (Column2, Column3)
SELECT n, REPlICATE('x', 2000) FROM #nums
DROP TABLE #nums
GO

-- Retrieve the inserted records
SELECT * FROM Table2
GO

-- Execute our "problematic" SQL statement in the 1st step.
-- Therefore SQL Server is able to produce and cache an suboptimal
-- Execution Plan, which will be reused afterwards.
SELECT * FROM Table1 t1
INNER HASH JOIN Table2 t2 ON t2.Column2 = t1.Column2
WHERE t1.Column2 = 2
GO

-- Insert 799 additional rows into the 1st table.
-- In that case SQL Server WILL NOT update the statistics,
-- because we have not hit the threshold of 800 data changes (20% + 500 rows).
SELECT TOP 799 IDENTITY(INT, 1, 1) AS n INTO #Nums
FROM
master.dbo.syscolumns sc1

INSERT INTO Table1 (Column2, Column3)
SELECT 2, REPLICATE('x', 2000) FROM #nums
DROP TABLE #nums
GO

-- Insert 799 additional rows into the 1st table.
-- In that case SQL Server WILL NOT update the statistics,
-- because we have not hit the threshold of 800 data changes (20% + 500 rows).
SELECT TOP 799 IDENTITY(INT, 1, 1) AS n INTO #Nums
FROM
master.dbo.syscolumns sc1

INSERT INTO Table2 (Column2, Column3)
SELECT 2, REPLICATE('x', 2000) FROM #nums
DROP TABLE #nums
GO

-- This query will produce a Hash Spill because of inaccurate statistics.
-- SQL Server estimates 1 row for the Hash Join, and requests a memory grant of 1 MB.
SELECT * FROM Table1 t1
INNER HASH JOIN Table2 t2 ON t2.Column2 = t1.Column2
WHERE t1.Column2 = 2
GO

-- Update the statistics on both tables.
-- Therefore SQL Server is able to estimate the memory grant correctly.
UPDATE STATISTICS Table1 WITH FULLSCAN
UPDATE STATISTICS Table2 WITH FULLSCAN
GO

-- The Hash Join is now running in memory.
-- SQL Server estimates now 800 rows, and requests a memory grant of 8.5 MB.
SELECT * FROM Table1 t1
INNER HASH JOIN Table2 t2 ON t2.Column2 = t1.Column2
WHERE t1.Column2 = 2
GO

-- Clean up
USE master
GO

DROP DATABASE HashSpills
GO